'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.isReactChildren = isReactChildren;
exports.createRouteFromReactElement = createRouteFromReactElement;
exports.createRoutesFromReactChildren = createRoutesFromReactChildren;
exports.createRoutes = createRoutes;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isValidChild(object) {
  return object == null || _react2.default.isValidElement(object);
}

function isReactChildren(object) {
  return isValidChild(object) || Array.isArray(object) && object.every(isValidChild);
}

function createRoute(defaultProps, props) {
  return _extends({}, defaultProps, props);
}

function createRouteFromReactElement(element) {
  const type = element.type;
  const route = createRoute(type.defaultProps, element.props);

  if (route.children) {
    const childRoutes = createRoutesFromReactChildren(route.children, route); // eslint-disable-line

    if (childRoutes.length) {
      route.childRoutes = childRoutes;
    }

    delete route.children;
  }

  return route;
}

function createRoutesFromReactChildren(children, parentRoute) {
  const routes = [];

  _react2.default.Children.forEach(children, element => {
    if (_react2.default.isValidElement(element)) {
      if (element.type.createRouteFromReactElement) {
        const route = element.type.createRouteFromReactElement(element, parentRoute);

        if (route) {
          routes.push(route);
        }
      } else {
        routes.push(createRouteFromReactElement(element));
      }
    }
  });

  return routes;
}

function createRoutes(routes) {
  if (isReactChildren(routes)) {
    routes = createRoutesFromReactChildren(routes); // eslint-disable-line
  } else if (routes && !Array.isArray(routes)) {
    routes = [routes]; // eslint-disable-line
  }

  return routes;
}